package com.common.commonlib.image.loader

import android.content.Context
import android.graphics.drawable.Drawable
import android.widget.ImageView
import com.bumptech.glide.Glide
import com.bumptech.glide.RequestBuilder
import com.bumptech.glide.load.resource.bitmap.BitmapTransformation
import com.bumptech.glide.load.resource.bitmap.GranularRoundedCorners
import com.common.commonlib.image.config.GlideBaseConfig
import com.common.commonlib.image.config.GlideHolderConfig
import com.common.commonlib.image.config.GlideTransitionConfig
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking

/**
 * 图片加载类
 */
object ImageLoader : ILoader {

    override fun init(
        context: Context, memoryCategory: GlideBaseConfig.MemoryType
    ) {
        Glide.get(context).setMemoryCategory(memoryCategory.memoryCategory)
    }

    fun setGlobalConfig(globalHolderConfig: GlideHolderConfig) {
        GlobalConfigHelper.globalHolderConfig = globalHolderConfig
    }

    fun setGlobalTransition(transitionConfig: GlideTransitionConfig) {
        GlobalConfigHelper.globalTransition = transitionConfig.transition
    }

    override fun load(context: Context, url: String?, view: ImageView) {
        loadWithMode(context, url, view, GlideBaseConfig.LoadMode.DEFAULT)
    }

    override fun loadRound(context: Context, url: String?, view: ImageView, radius: Float) {
        var request = GlobalConfigHelper.loadWithGlobalConfig(Glide.with(context).load(url))
        request = ConfigHelper.loadWithMod(request, GlideBaseConfig.LoadMode.DEFAULT)

        val transformations = ArrayList<BitmapTransformation>()
        transformations.add(GranularRoundedCorners(radius, radius, radius, radius))
        request = ConfigHelper.loadWithTransformation(request, transformations)
        loadIntoView(request, view)
    }

    override fun loadWithMode(
        context: Context,
        url: String?,
        view: ImageView,
        loadingMode: GlideBaseConfig.LoadMode
    ) {
        var request = GlobalConfigHelper.loadWithGlobalConfig(Glide.with(context).load(url))
        request = ConfigHelper.loadWithMod(request, loadingMode)
        loadIntoView(request, view)
    }

    override fun loadWithPlaceHolder(
        context: Context,
        url: String,
        view: ImageView,
        placeGlideHolder: GlideHolderConfig
    ) {
        loadIntoView(
            GlobalConfigHelper.loadWithGlobalConfig(
                Glide.with(context).load(url),
                placeGlideHolder
            ), view
        )
    }

    private fun loadIntoView(requestBuilder: RequestBuilder<Drawable>, view: ImageView) {
        requestBuilder.into(view)
    }

    override fun clearMemoryCache(context: Context){
        GlobalScope.launch(Dispatchers.Main) {
            Glide.get(context).clearMemory()
        }
    }

    override fun clearDiskCache(context: Context) = runBlocking<Unit> {
        launch(Dispatchers.Default) {
            Glide.get(context).clearDiskCache()
        }
    }

    override fun clearAllCache(context: Context) = runBlocking<Unit> {
        GlobalScope.launch(Dispatchers.Main) {
            Glide.get(context).clearMemory()
        }
        launch(Dispatchers.Default) {
            Glide.get(context).clearDiskCache()
        }
    }
}